////////////////////////////////////////////////////////////////////////////
// chem2deditor.cpp
// (c) Left 2000 Ricardo Stefani
// This source File is part of chem2D
///////////////////////////////////////////////////////////////////////////

#include "chem2Deditor.h"
#include <qcanvas.h>
#include <qpointarray.h>
#include <qpoint.h>
#include <qinputdialog.h>
#include <qfontdialog.h>
#include <qmessagebox.h>

#include "chemhexane.h"
#include "chemaromatic.h"
#include "chemcpentane.h"
#include "chembond.h"
#include "chemPenDlg.h"

bool operator<=(const QPoint& p1, const QPoint& p2)
{
return( (p1.x() <= p2.x() ) && (p1.y() <= p2.y() ) );
}

bool operator>=(const QPoint& p1, const QPoint& p2)
{
return((p1.x() >= p2.x()) && (p1.y() >= p2.y()));
}


chem2DEditor::~chem2DEditor()
{
 if(cv)
   delete cv;
}

chem2DEditor::chem2DEditor(QCanvas* v, QWidget* parent,
                           const char* name, WFlags f) :
          QCanvasView(v, parent, name, f)
{
  if(!v)
   {
   cv = new QCanvas(1170,800);
   setCanvas(cv);
   }
}

void chem2DEditor::contentsMousePressEvent(QMouseEvent *ev)
{
   
   if(ev->button() == LeftButton) // Left Button Click
   {
        ActualItem = 0;

        if(ev->state() & ShiftButton)
         { // LeftButton Shift
          drawing = false;
          SelectItem(ev);
          chemBond* i = dynamic_cast<chemBond*> (ActualItem);
          if(i)
          i->setStyle(i->getNextstyle());
          cv->update();
          return;
         }

        start = ev->pos();
        
        if(!ActualItem)
        {
         drawing = true;
          
         switch(Selected)
           {
            case SELECT_TOOL:
              {
              drawing = false;
              SelectItem(ev);
              break;
              }
             case LINE_TOOL:
               {
                  DrawBond(ev);
                     break;
               }
             case SBUNDER_TOOL:
              {
                   DrawSbunder(ev);
                   break;
              }
             case SBHIGH_TOOL:
              {
                  DrawSbhigh(ev);
                  break;
              }
             case TEXT_TOOL:
              {
                 DrawAtom(ev);
                 break;
              }
            case CPENTANE_TOOL:
              {
                 DrawCpentane(ev);
                 break;
              }
            case CICLE_TOOL:
             {
                DrawCicle(ev);
                break;
             }
            case AROMA_TOOL:
             {
               DrawAromatic(ev);
               break;
             }
        }
      cv->update();
     }
  } else if(ev->button() & RightButton) // elimina Item
       if(ev->state() & ControlButton)
          {
          drawing = false;
          SelectItem(ev);
          delete ActualItem;
          cv->update();
	  } else ProcessPopup(ev);
          
}

void chem2DEditor::contentsMouseMoveEvent(QMouseEvent* ev)
{
    if ( ActualItem && !drawing) {
         ActualItem->moveBy(ev->pos().x() - start.x(),
		               ev->pos().y() - start.y());
	 start = ev->pos();
	 cv->update();
    } else
    if( ActualItem && drawing ) //It's not good but assuming only lines....
      {
        if((Selected == LINE_TOOL) || (Selected == SBUNDER_TOOL) ||
           (Selected == SBHIGH_TOOL)) // Change Line
         {
         chemBond *i = dynamic_cast<chemBond*> (ActualItem);
         if(i)
         i->setPoints(0,0,ev->pos().x() - start.x(),
                   ev->pos().y() - start.y());
         cv->update();
         }

      }



}

void chem2DEditor::contentsMouseReleaseEvent(QMouseEvent* ev)
{
 if(ev->button() == LeftButton)
   if(ActualItem && drawing)
     drawing = false;

}

// double clicks

void chem2DEditor::contentsMouseDoubleClickEvent(QMouseEvent* ev)
{
  if(ev->button() == LeftButton)
    {
     SelectItem(ev);
     if(ActualItem)
       {
       chemHexane *hx = dynamic_cast<chemHexane*> (ActualItem);
       if(hx)
         {

         QPointArray pa = hx->realPoints();
         // 3 pixels tolerancy
         if( ((ev->x() >= (pa[1].x() - 3)) && (ev->x() <= (pa[1].x() + 3))) &&
             ((ev->x() >= (pa[2].x() - 3)) && (ev->x() <= (pa[2].x() + 3))))
              {
              hx->changeOrder(QPoint(pa[1].x() + 3, pa[1].y() - 2),
                              QPoint(pa[2].x() + 3, pa[2].y() + 2));
              cv->update();
              }
         //
         if( ((ev->x() >= (pa[4].x() - 3)) && (ev->x() <= (pa[4].x() + 3))) &&
             ((ev->x() >= (pa[5].x() - 3)) && (ev->x() <= (pa[5].x() + 3))))
              {
              hx->changeOrder(QPoint(pa[4].x() - 3, pa[4].y() + 2),
                              QPoint(pa[5].x() - 3, pa[5].y() - 2));
              cv->update();
              }

         if((ev->pos() <= pa[0]) && (ev->pos() >= pa[1]))
             {
              hx->changeOrder(QPoint(pa[0].x() - 2, pa[0].y() - 5),
                              QPoint(pa[1].x() + 4, pa[1].y() - 2));
              cv->update();
              }

         if((ev->pos() >= pa[0]) && (ev->pos() <= pa[5]))
            {
              QMessageBox::information(0, "Info", "It works !");
              hx->changeOrder(QPoint(pa[0].x() - 2, pa[0].y() - 5),
                              QPoint(pa[1].x() + 4, pa[1].y() - 2));
              cv->update();
              }     

         }
       }
    }
}

// popupMenu

void chem2DEditor::ProcessPopup(QMouseEvent* ev)
{
drawing = false;
SelectItem(ev);

if(ActualItem)
{
   if(ActualItem->rtti()==3) // se for Texto
      {
       menuProperties = new QPopupMenu(viewport());
       menuProperties->insertItem( tr("Change Attributes..."), this, SLOT(setFont()) );
      } else
      {
       menuProperties = new QPopupMenu;
       menuProperties->insertItem( tr("Change Attributes..."), this, SLOT(setPen()) );
      }
menuProperties->popup(ev->pos());
}


}

// popupmenu Commands

void chem2DEditor::setPen()
{
 if(!ActualItem)
    return;

 chemPenDlg *dlg = new chemPenDlg(this);
 QCanvasPolygonalItem *it;
 if(dlg->exec())
  {
    it = dynamic_cast<QCanvasPolygonalItem*> (ActualItem);
    if(it)
     it->setPen( dlg->pen );
    cv->update();
  }
}

void chem2DEditor::setFont()
{
bool ok;
if(!ActualItem)
 return;
QCanvasText *tx = dynamic_cast<QCanvasText*> (ActualItem);
if(tx)
  {
   QFont font = QFontDialog::getFont(&ok);
   if(ok)
    tx->setFont( font );
    cv->update();
  }
}

// Draw Tools

void chem2DEditor::SelectItem(QMouseEvent* ev)
{
 QCanvasItem* itTemp; // temporary Item
 ActualItem = 0;

 QCanvasItemList li = cv->allItems(); // Find the Item to Select
   for (QCanvasItemList::Iterator iti = li.begin(); iti!=li.end(); ++iti)
     {
      itTemp = *iti;
      itTemp->setSelected(false);
     } 


 QCanvasItemList l = cv->collisions(ev->pos()); // Find the Item to Select
   for (QCanvasItemList::Iterator it = l.begin(); it!=l.end(); ++it)
     {
      ActualItem = *it;
      ActualItem->setSelected(true);
      start = ev->pos();
     }

}


void chem2DEditor::DrawBond(QMouseEvent* ev)
{
  chemBond *i = new chemBond(cv, SINGLE_BOND);
  i->setPoints(0,0,ev->pos().x() - start.x(),
                   ev->pos().y() - start.y());
  i->move(ev->x(),ev->y());
  ActualItem = i;
}

void chem2DEditor::DrawAtom(QMouseEvent* ev)
{
  QString symbol = QInputDialog::getText( tr("Atom Symbol"), tr("Enter an atom symbol"),
                                 QString::null);
  QCanvasText *i = new QCanvasText(symbol, cv);
  i->move(ev->x(),ev->y());
  drawing = false;
}

void chem2DEditor::DrawCicle(QMouseEvent* ev)
{
   
    chemHexane* i = new chemHexane(cv);
    const int size = 20;
    QPointArray px(6);
    px[0] = QPoint(0,2*size);
    px[1] = QPoint(-size*173/100,size);
    px[2] = QPoint(-size*173/100,-size);
    px[3] = QPoint(0,-2*size);
    px[4] = QPoint(size*173/100,-size);
    px[5] = QPoint(size*173/100,size);

    i->setPoints(px);
    i->move(ev->x(), ev->y());

    drawing = false;
}

void chem2DEditor::DrawAromatic(QMouseEvent* ev)
{
    chemAromatic* i = new chemAromatic(cv);
    const int size = 20;
    QPointArray px(6);

    px[0] = QPoint(0,2*size);
    px[1] = QPoint(-size*173/100,size);
    px[2] = QPoint(-size*173/100,-size);
    px[3] = QPoint(0,-2*size);
    px[4] = QPoint(size*173/100,-size);
    px[5] = QPoint(size*173/100,size);

    i->setPoints(px);
    i->move(ev->x(), ev->y());
    drawing = false;
}

void chem2DEditor::DrawCpentane(QMouseEvent *ev)
{
    chemCpentane* i = new chemCpentane(cv);
    const int size = 20;
    QPointArray px(5);
    px[0] = QPoint(0,-2*size);
    px[1] = QPoint(-size*173/100,-size);
    px[2] = QPoint(-size*173/100,size);
    px[3] = QPoint(size*173/100,size);
    px[4] = QPoint(size*173/100,-size);

    i->setPoints(px);
    i->move(ev->x(), ev->y());
    drawing = false;
}


void chem2DEditor::DrawSbunder(QMouseEvent* ev)
{
    chemBond* i = new chemBond(cv, UPLA_BOND);
    i->setPoints(0,0,ev->pos().x() - start.x(),
                   ev->pos().y() - start.y());
    i->move(ev->x(),ev->y());
    ActualItem = i;
}

void chem2DEditor::DrawSbhigh(QMouseEvent* ev)
{
    chemBond* i = new chemBond(cv, HPLA_BOND);
    i->setPoints(0,0,ev->pos().x() - start.x(),
                   ev->pos().y() - start.y());
    i->move(ev->x(),ev->y());
    ActualItem = i;
}
