///////////////////////////////////////////////////////////////////////////
// chemhexane.cpp
///////////////////////////////////////////////////////////////////////////

#include "chembond.h"

chemBond::chemBond(QCanvas* canvas) :
    QCanvasPolygonalItem(canvas)
{
style = SINGLE_BOND;
x1 = x2 = y1 = y2 = 0;
setPen( QPen( QColor(0,0,0), 1) );
}

chemBond::chemBond(QCanvas* canvas, BondStyle style) :
         QCanvasPolygonalItem(canvas)
{
this->style = style;
x1 = x2 = y1 = y2 = 0;
setPen( QPen( QColor(0,0,0), 1) );
}

chemBond::~chemBond()
{
    hide();
}

void chemBond::drawShape(QPainter& p)
{
   if(style == SINGLE_BOND)
    p.drawLine((int)(x()+x1), (int)(y()+y1), (int)(x()+x2), (int)(y()+y2));

   if(style == DOUBLE_BOND)
    {
    p.drawLine((int)(x()+x1), (int)(y()+y1), (int)(x()+x2), (int)(y()+y2));
    if(x1 == x2)
      p.drawLine((int)(x()+x1+3), (int)(y()+y1), (int)(x()+x2+3), (int)(y()+y2));
    if(y1 == y2)
      p.drawLine((int)(x()+x1), (int)(y()+y1+3), (int)(x()+x2), (int)(y()+y2+3));
    if((x1 < x2) && (y1 > y2))
      p.drawLine((int)(x()+x1), (int)(y()+y1-3), (int)(x()+x2), (int)(y()+y2-3));
    if((x1 > x2) && (y2 > y1))
      p.drawLine((int)(x()+x1), (int)(y()+y1-3), (int)(x()+x2), (int)(y()+y2-3));
    if((x1 > x2) && (y1 > y2))
      p.drawLine((int)(x()+x1), (int)(y()+y1+3), (int)(x()+x2), (int)(y()+y2+3));
    if((x1 < x2) && (y1 < y2))
      p.drawLine((int)(x()+x1), (int)(y()+y1+3), (int)(x()+x2), (int)(y()+y2+3));

    }

    if(style == TRIPLE_BOND)
    {
    p.drawLine((int)(x()+x1), (int)(y()+y1), (int)(x()+x2), (int)(y()+y2));
    if(x1 == x2)
      {
      p.drawLine((int)(x()+x1+3), (int)(y()+y1), (int)(x()+x2+3), (int)(y()+y2));
      p.drawLine((int)(x()+x1-3), (int)(y()+y1), (int)(x()+x2-3), (int)(y()+y2));
      }
    if(y1 == y2)
      {
      p.drawLine((int)(x()+x1), (int)(y()+y1+3), (int)(x()+x2), (int)(y()+y2+3));
      p.drawLine((int)(x()+x1), (int)(y()+y1-3), (int)(x()+x2), (int)(y()+y2-3));
      }
    if((x1 < x2) && (y1 > y2))
      {
      p.drawLine((int)(x()+x1), (int)(y()+y1-3), (int)(x()+x2), (int)(y()+y2-3));
      p.drawLine((int)(x()+x1), (int)(y()+y1+3), (int)(x()+x2), (int)(y()+y2+3));
      }
    if((x1 > x2) && (y2 > y1))
      {
      p.drawLine((int)(x()+x1), (int)(y()+y1-3), (int)(x()+x2), (int)(y()+y2-3));
      p.drawLine((int)(x()+x1), (int)(y()+y1+3), (int)(x()+x2), (int)(y()+y2+3));
      }
    if((x1 > x2) && (y1 > y2))
      {
      p.drawLine((int)(x()+x1), (int)(y()+y1+3), (int)(x()+x2), (int)(y()+y2+3));
      p.drawLine((int)(x()+x1), (int)(y()+y1-3), (int)(x()+x2), (int)(y()+y2-3));
      }
    if((x1 < x2) && (y1 < y2))
      {
      p.drawLine((int)(x()+x1), (int)(y()+y1+3), (int)(x()+x2), (int)(y()+y2+3));
      p.drawLine((int)(x()+x1), (int)(y()+y1-3), (int)(x()+x2), (int)(y()+y2-3));
      }
    }
  if(style == UPLA_BOND)
    {
    pp = QPointArray(3);
    pp[0] =  QPoint((int)(x()+x1), (int)(y()+y1));
    pp[1] =  QPoint((int)(x()+x2), (int)(y()+y2));
    setBrush( QBrush( NoBrush ) );
    p.drawLine((int)(x()+x1), (int)(y()+y1), (int)(x()+x2), (int)(y()+y2));
    if(x1 == x2)
      pp[2] = QPoint((int)(x()+x2+6), (int)(y()+y2));
    if(y1 == y2)
      pp[2] = QPoint((int)(x()+x2), (int)(y()+y2+6));
    if((x1 < x2) && (y1 > y2))
      pp[2] = QPoint((int)(x()+x2), (int)(y()+y2-6));
    if((x1 > x2) && (y2 > y1))
      pp[2] = QPoint((int)(x()+x2), (int)(y()+y2-6));
    if((x1 > x2) && (y1 > y2))
      pp[2] = QPoint((int)(x()+x2), (int)(y()+y2+6));
    if((x1 < x2) && (y1 < y2))
      pp[2] = QPoint((int)(x()+x2), (int)(y()+y2+6));

    p.drawPolygon(pp);
    }

   if(style == HPLA_BOND)
    {
    pp = QPointArray(3);
    pp[0] =  QPoint((int)(x()+x1), (int)(y()+y1));
    pp[1] =  QPoint((int)(x()+x2), (int)(y()+y2));
    setBrush( QBrush( pen().color() , SolidPattern ) );
    p.drawLine((int)(x()+x1), (int)(y()+y1), (int)(x()+x2), (int)(y()+y2));
    if(x1 == x2)
      pp[2] = QPoint((int)(x()+x2+6), (int)(y()+y2));
    if(y1 == y2)
      pp[2] = QPoint((int)(x()+x2), (int)(y()+y2+6));
    if((x1 < x2) && (y1 > y2))
      pp[2] = QPoint((int)(x()+x2), (int)(y()+y2-6));
    if((x1 > x2) && (y2 > y1))
      pp[2] = QPoint((int)(x()+x2), (int)(y()+y2-6));
    if((x1 > x2) && (y1 > y2))
      pp[2] = QPoint((int)(x()+x2), (int)(y()+y2+6));
    if((x1 < x2) && (y1 < y2))
      pp[2] = QPoint((int)(x()+x2), (int)(y()+y2+6));

    p.drawPolygon(pp);
    } 

    
}

void chemBond::setPoints(int x1, int y1, int x2, int y2)
{
 hide();
 this->x1 = x1;
 this->x2 = x2;
 this->y1 = y1;
 this->y2 = y2;
 show();
}

void chemBond::moveBy(double dx, double dy)
{
        hide();
	QCanvasItem::moveBy(dx,dy);
        show();
}


QPointArray chemBond::points() const
{
   QPointArray p(4);
    int xi = int(x());
    int yi = int(y());
    int pw = pen().width();
    int dx = QABS(x1-x2);
    int dy = QABS(y1-y2);
    if ( pw <= 1 ) {
	// thin
	if ( dx > dy ) {
	    p[0] = QPoint(x1+xi,y1+yi-9);
	    p[1] = QPoint(x2+xi,y2+yi-9);
	    p[2] = QPoint(x2+xi,y2+yi+9);
	    p[3] = QPoint(x1+xi,y1+yi+9);
	} else {
	    p[0] = QPoint(x1+xi-9,y1+yi);
	    p[1] = QPoint(x2+xi-9,y2+yi);
	    p[2] = QPoint(x2+xi+9,y2+yi);
	    p[3] = QPoint(x1+xi+9,y1+yi);
	}
    } else {
	pw = pw*4/3+2; // approx pw*sqrt(2)
	int px = x1<x2 ? -pw : pw ;
	int py = y1<y2 ? -pw : pw ;
	if ( dx && dy && (dx > dy ? (dx*9/dy <= 9) : (dy*9/dx <= 9)) ) {
	    // steep
	    if ( px == py ) {
		p[0] = QPoint(x1+xi   ,y1+yi+py);
		p[1] = QPoint(x2+xi-px,y2+yi   );
		p[2] = QPoint(x2+xi   ,y2+yi-py);
		p[3] = QPoint(x1+xi+px,y1+yi   );
	    } else {
		p[0] = QPoint(x1+xi+px,y1+yi   );
		p[1] = QPoint(x2+xi   ,y2+yi-py);
		p[2] = QPoint(x2+xi-px,y2+yi   );
		p[3] = QPoint(x1+xi   ,y1+yi+py);
	    }
	} else if ( dx > dy ) {
	    // horizontal
	    p[0] = QPoint(x1+xi+px,y1+yi+py);
	    p[1] = QPoint(x2+xi-px,y2+yi+py);
	    p[2] = QPoint(x2+xi-px,y2+yi-py);
	    p[3] = QPoint(x1+xi+px,y1+yi-py);
	} else {
	    // vertical
	    p[0] = QPoint(x1+xi+px,y1+yi+py);
	    p[1] = QPoint(x2+xi+px,y2+yi-py);
	    p[2] = QPoint(x2+xi-px,y2+yi-py);
	    p[3] = QPoint(x1+xi-px,y1+yi+py);
	}
    }
    return p;
}

QPointArray chemBond::areaPoints() const
{
 return(points());
}

BondStyle chemBond::getNextstyle()
{
 if(style==HPLA_BOND)
    return SINGLE_BOND;
 else if (style==SINGLE_BOND)
    return DOUBLE_BOND;
 else if (style==DOUBLE_BOND)
    return TRIPLE_BOND;
 else if(style==UPLA_BOND)
    return HPLA_BOND;
}
