/****************************************************************************
** $Id: qt/examples/mdi/application.cpp   2.2.0-beta2   edited 2000-08-05 $
**
** Copyright (C) 1992-2000 Troll Tech AS.  All rights reserved.
**
** This file is part of an example program for Qt.  This example
** program may be used, distributed and modified without limitation.
**
*****************************************************************************/

#include "application.h"
#include "spview.h"
#include <qworkspace.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qfile.h>
#include <qfiledialog.h>
#include <qlabel.h>
#include <qstatusbar.h>
#include <qmessagebox.h>
#include <qprinter.h>
#include <qapplication.h>
#include <qpushbutton.h>
#include <qaccel.h>
#include <qtextstream.h>
#include <qwhatsthis.h>
#include <qobjectlist.h>
#include <qvbox.h>
#include <qbuttongroup.h>
#include <qwidget.h>
#include <qcursor.h>


#include "filesave.xpm"
#include "fileopen.xpm"
#include "fileprint.xpm"

ApplicationWindow::ApplicationWindow()
    : QMainWindow( 0, "chemIR", WDestructiveClose )
{
    int id;

    QPixmap openIcon, saveIcon;

    fileTools = new QToolBar( this, tr("file operations") );
    addToolBar( fileTools, tr( "File Operations" ), Top, TRUE );

    openIcon = QPixmap( fileopen );
    QToolButton * fileOpen
	= new QToolButton( openIcon, tr("Open File"), QString::null,
			   this, SLOT(load()), fileTools, tr("open file") );

    saveIcon = QPixmap( filesave );
    QToolButton * fileSave
	= new QToolButton( saveIcon, tr("Save File"), QString::null,
			   this, SLOT(save()), fileTools, tr("save file") );

#ifndef QT_NO_PRINTER
    printer = new QPrinter;
    QPixmap printIcon;

    printIcon = QPixmap( fileprint );
    QToolButton * filePrint
	= new QToolButton( printIcon, tr("Print File"), QString::null,
			   this, SLOT(print()), fileTools, tr("print file") );
#endif


    QPopupMenu * file = new QPopupMenu( this );
    menuBar()->insertItem( tr("&File"), file );

    id = file->insertItem( openIcon, tr("&Open"),
			   this, SLOT(load()), CTRL+Key_O );

    id = file->insertItem( saveIcon, tr("&Save"),
			   this, SLOT(save()), CTRL+Key_S );
    id = file->insertItem( tr("Save &as..."), this, SLOT(saveAs()) );
#ifndef QT_NO_PRINTER
    file->insertSeparator();
    id = file->insertItem( printIcon, tr("&Print"),
			   this, SLOT(print()), CTRL+Key_P );
#endif
    file->insertSeparator();
    file->insertItem( tr("&Close"), this, SLOT(closeWindow()), CTRL+Key_W );
    file->insertItem( tr("&Quit"), qApp, SLOT( closeAllWindows() ), CTRL+Key_Q );

    windowsMenu = new QPopupMenu( this );
    windowsMenu->setCheckable( TRUE );
    connect(windowsMenu, SIGNAL(aboutToShow()),
                        SLOT(windowsMenuAboutToShow()));
    menuBar()->insertItem( tr("&Windows"), windowsMenu );

    menuBar()->insertSeparator();
    QPopupMenu * help = new QPopupMenu( this );
    menuBar()->insertItem( tr("&Help"), help );

    help->insertItem( tr("&About"), this, SLOT(about()), Key_F1);
    help->insertItem( tr("About&Qt"), this, SLOT(aboutQt()));
    help->insertSeparator();

    QVBox* vb = new QVBox( this );
    vb->setFrameStyle( QFrame::StyledPanel | QFrame::Sunken );
    ws = new QWorkspace( vb );
    setCentralWidget( vb );

    statusBar()->message( tr("Ready"), 2000 );

}


ApplicationWindow::~ApplicationWindow()
{
#ifndef QT_NO_PRINTER
    delete printer;
#endif
}


void ApplicationWindow::load()
{
  QString fn = QFileDialog::getOpenFileName( QString::null, tr("Perkin Elmer Data (*.sp)"), this );
    if ( !fn.isEmpty() ) {
	spView *w  = new spView(ws);
	w->load( fn );
        w->show();
    }  else {
	statusBar()->message( tr("Loading aborted"), 2000 );
    }
}

void ApplicationWindow::save()
{
  /*  chem2DEditor* m = (chem2DEditor*)ws->activeWindow();
    if ( m )
	m->save();*/
}


void ApplicationWindow::saveAs()
{
  /*  chem2DEditor* m = (chem2DEditoe*)ws->activeWindow();
    if ( m )
	m->saveAs();*/
}


void ApplicationWindow::print()
{
}


void ApplicationWindow::closeWindow()
{
  /*  chem2DEditor* m = (chem2DEditor*)ws->activeWindow();
    if ( m )
	m->close();*/
}

void ApplicationWindow::about()
{
    QMessageBox::about( this, tr("ChemIR - Chemsuite"),
			tr("ChemIR - Chemsuite Development\n "
			"This program is part of Chemsuite."));
}


void ApplicationWindow::aboutQt()
{
    QMessageBox::aboutQt( this, "Chemsuite chemIR" );
}


void ApplicationWindow::windowsMenuAboutToShow()
{
    windowsMenu->clear();
    int cascadeId = windowsMenu->insertItem(tr("&Cascade"), ws, SLOT(cascade() ) );
    int tileId = windowsMenu->insertItem(tr("&Tile"), ws, SLOT(tile() ) );
    if ( ws->windowList().isEmpty() ) {
	windowsMenu->setItemEnabled( cascadeId, FALSE );
	windowsMenu->setItemEnabled( tileId, FALSE );
    }
    windowsMenu->insertSeparator();
    QWidgetList windows = ws->windowList();
    for ( int i = 0; i < int(windows.count()); ++i ) {
	int id = windowsMenu->insertItem(windows.at(i)->caption(),
					 this, SLOT( windowsMenuActivated( int ) ) );
	windowsMenu->setItemParameter( id, i );
	windowsMenu->setItemChecked( id, ws->activeWindow() == windows.at(i) );
    }
}

void ApplicationWindow::windowsMenuActivated( int id )
{
    QWidget* w = ws->windowList().at( id );
    if ( w )
	w->setFocus();
}



